@@ -39,6 +39,8 @@
 
 ;;; Code:
 
+(use-modules (ice-9 threads))
+
 ;;
 ;; Game Options
 ;;
@@ -182,6 +184,12 @@
 ;; How to move cards
 ;;
 
+(define (movable-to-homecell? card-list homecell-id)
+  (and (= (length card-list) 1)
+       (if (empty-slot? homecell-id)
+           (eq? (get-value (car card-list)) ace)
+           (homecell-join? (car (get-cards homecell-id)) (car card-list)))))
+
 (define (move-to-homecell card-list homecell-id)
 	(and
 		(= (length card-list) 1)
@@ -211,19 +219,27 @@
 	)
 )
 
-(define (move-to-field start-slot card-list field-id)
+;; Version of move-to-field that only tests a move or supermove.
+(define (movable-to-field? start-slot card-list field-id)
   (and (field-sequence? card-list)
-       (<= (length card-list) 
-	   (* (+ (empty-freecell-number) 1)
-	      ($expt 2 (max (- (empty-field-number)
-			       (if (empty-slot? field-id) 1 0)
-			       (if (empty-slot? start-slot) 1 0))
-			    0))))
-       (if (empty-slot? field-id)
-	   (add-cards! field-id card-list)
-	   (let ((dest-top (car (get-cards field-id))))
-	     (and (field-sequence? (append card-list (list dest-top)))
-		  (add-cards! field-id card-list))))))
+       (<= (length card-list)
+           (* (+ (empty-freecell-number) 1)
+              ($expt 2 (max (- (empty-field-number)
+                               (if (empty-slot? field-id) 1 0)
+                               (if (empty-slot? start-slot) 1 0))
+                            0))))
+       (or (empty-slot? field-id)
+           (let ((dest-top (car (get-cards field-id))))
+             (and (field-sequence? (append card-list (list dest-top))))))))
+
+
+(define (move-to-field start-slot card-list field-id)
+  (and (movable-to-field? start-slot card-list field-id)
+       (add-cards! field-id card-list)))
+
+(define (movable-to-freecell? card-list freecell-id)
+  (and (= (length card-list) 1)
+       (empty-slot? freecell-id)))
 
 (define (move-to-freecell card-list freecell-id)
 	(and
@@ -314,33 +330,32 @@
 
   ; freecells
   (add-normal-slot '())			; 0
+  (set! HORIZPOS (- HORIZPOS (/ 1 24)))
   (add-normal-slot '())			; 1
+  (set! HORIZPOS (- HORIZPOS (/ 1 24)))
   (add-normal-slot '())			; 2
+  (set! HORIZPOS (- HORIZPOS (/ 1 24)))
   (add-normal-slot '())			; 3
   (set! HORIZPOS (+ HORIZPOS 0.25))
 
   ; homecells
   (add-normal-slot '())			; 4
+  (set! HORIZPOS (- HORIZPOS (/ 1 24)))
   (add-normal-slot '())			; 5
+  (set! HORIZPOS (- HORIZPOS (/ 1 24)))
   (add-normal-slot '())			; 6
+  (set! HORIZPOS (- HORIZPOS (/ 1 24)))
   (add-normal-slot '())			; 7
   (add-carriage-return-slot)
 
   ; fields
   (add-extended-slot '() down)		; 8
-  (set! HORIZPOS (+ HORIZPOS (/ 1 28)))
   (add-extended-slot '() down)		; 9
-  (set! HORIZPOS (+ HORIZPOS (/ 1 28)))
   (add-extended-slot '() down)		; 10
-  (set! HORIZPOS (+ HORIZPOS (/ 1 28)))
   (add-extended-slot '() down)		; 11
-  (set! HORIZPOS (+ HORIZPOS (/ 1 28)))
   (add-extended-slot '() down)		; 12
-  (set! HORIZPOS (+ HORIZPOS (/ 1 28)))
   (add-extended-slot '() down)		; 13
-  (set! HORIZPOS (+ HORIZPOS (/ 1 28)))
   (add-extended-slot '() down)		; 14
-  (set! HORIZPOS (+ HORIZPOS (/ 1 28)))
   (add-extended-slot '() down)		; 15
 
   (add-blank-slot)
@@ -351,8 +366,10 @@
   (update-auto spade 0)
 
   (set! board-hash (make-hash-table hash-size))
+  
+  (start-hint-thread)
 
-  (list 8.25 3)
+  (list 8 3.5)
 )
 
 (define (button-pressed slot card-list)
@@ -360,7 +377,16 @@
 	((field?      slot) (field-sequence? card-list))
 	((freecell?   slot) #t)))
 
+(define (droppable? start-slot card-list end-slot)
+        (and (not (= start-slot end-slot))
+             (cond
+               ((homecell? end-slot) (movable-to-homecell? card-list end-slot))
+               ((field?    end-slot) (movable-to-field? start-slot card-list end-slot))
+	       ((freecell? end-slot) (movable-to-freecell? card-list end-slot))
+               (else #f))))
+
 (define (button-released start-slot card-list end-slot)
+  (kill-hint-thread)
 	(and
 		(not (= start-slot end-slot))
 		(cond
@@ -370,6 +396,7 @@
 		)
 		(move-low-cards 0)
 	)
+	(start-hint-thread)
 )
   
 (define (button-clicked slot)
@@ -377,17 +404,19 @@
   #f)
 
 (define (button-double-clicked slot)
+  (kill-hint-thread)
 	(and
 		(not (empty-slot? slot))
 		(let ((card (get-top-card slot)))
 			(and
-;				(move-card-to-homecell card (homecell-by-suit (get-suit card)))
-				(move-card-to-freecell card (any-empty-freecell))
+				(or (move-card-to-homecell card (homecell-by-suit (get-suit card)))
+				    (move-card-to-freecell card (any-empty-freecell)))
 				(remove-card slot)
 				(move-low-cards 0)
 			)
 		)
 	)
+	(start-hint-thread)
 )
 
 ;; Condition for fail -- no more cards to move
@@ -525,7 +554,7 @@
 ; These constants affect the hash table and search algorithm
 (define hash-size (- (expt 2 17) 1)) ; A Mersenne prime (2^17 - 1) ~128k
 (define board-node-max  50) ; number of board positions to visit each time.
-(define traverse-node-max 1000) ; prevents stack overflow
+(define traverse-node-max 10000) ; prevents stack overflow
 
 ; These constants define values used in constructing the board value.
 (define weight-factor   100) ; effect of weight on final score
@@ -547,11 +576,16 @@
 (define visited-nodes 0) ; Number of board positions created for this search.
 (define traversed-nodes 0) ; Number of board positions traversed through
 
+(define hint-return '()) ; The hint, updated by the hint thread
+(define kill-thread #f) ; Tell the hint thread to die
+(define kill-thread-mutex (make-mutex))
+(define hint-thread '()) 
+
 ;;-----------------------------------------------------------------------------
 ;; Functions
 
 ; Returns the best move found by the search algorithm
-(define (get-hint)
+(define (find-hint)
   (if debug (display "get-hint\n"))
   (set! visited-nodes 0)
   (set! traversed-nodes 0)
@@ -570,6 +604,39 @@
 	  (display-best-move-trace board moves)))
       (create-help-list board moves))))
 
+; Start the hint thread on its way
+(define (start-hint-thread)
+  (if debug (display "Starting hint thread\n"))
+  (set! hint-return (list 0 (_"No hint available right now")))
+  (set! kill-thread #f) ; No locking, we haven't started the thread yet
+  (set! hint-thread
+	(begin-thread 
+	 (set! hint-return (find-hint)))))
+
+; Test the kill-thread flag
+(define (kill-thread?)
+  (lock-mutex kill-thread-mutex)
+  (let ((result kill-thread))
+    (unlock-mutex kill-thread-mutex)
+    result))
+		      
+; Kill the hint thread and ignore the result
+(define (kill-hint-thread)
+  (if debug (display "Killing hint thread\n"))
+  (lock-mutex kill-thread-mutex)
+  (set! kill-thread #t)             ; Signal the kill
+  (unlock-mutex kill-thread-mutex)
+  (join-thread hint-thread))        ; and wait for the code to unwind
+
+; The hint callback. It stops the hint thread and gets the current best 
+; result. Note that the thread is not restarted since we let the hint code 
+; unwind and we would therefore have to start at the beginning. In practise 
+; the hint code is faster than the time it takes for the player to click the 
+; hint button so it doesn't matter.
+(define (get-hint)
+  (kill-hint-thread)
+  hint-return)
+
 ; Displays the sequence of best moves found so far by the search. (Debug only)
 ; Note that the best sequence is occasionally not available depending on
 ; how the hint function terminates the search.  In these cases, this function
@@ -669,7 +736,8 @@
 	     (not (eq? (vector-ref (cdaar moves) index-outcome) outcome-lose)))
 
       ; Determine whether to traverse deeper, or to go back up the tree
-      (if (and (eq? (vector-ref (cdaar moves) index-outcome) #f)
+      (if (and (kill-thread?)
+	       (eq? (vector-ref (cdaar moves) index-outcome) #f)
 	       (< visited-nodes board-node-max)
 	       (< traversed-nodes traverse-node-max)
 	       (>= (vector-ref (cdaar moves) index-value) prev-best))
@@ -991,7 +1059,9 @@
     (+ (get-board-free-count board (cdr cells))
        (if (null? (vector-ref board (car cells))) 1 0))))
 
-(set-lambda new-game button-pressed button-released button-clicked button-double-clicked game-over game-won get-hint get-options apply-options timeout)
+(set-features droppable-feature)
+
+(set-lambda new-game button-pressed button-released button-clicked button-double-clicked game-over game-won get-hint get-options apply-options timeout droppable?)
 
 ;;; freecell.scm ends here
 
