@@ -110,11 +110,11 @@ message_box (gchar *msg)
 {
   GtkWidget *box;
 
-  box = gnome_app_message (GNOME_APP (app),
-                           msg);
-
-  gtk_window_set_modal (GTK_WINDOW (box), TRUE);
-  gtk_widget_show (box);
+  box = gtk_message_dialog_new (GTK_WINDOW (app), GTK_DIALOG_MODAL, 
+				GTK_MESSAGE_INFO, GTK_BUTTONS_OK,
+				msg);
+  gtk_dialog_run (GTK_DIALOG (box));
+  gtk_widget_destroy (box);
 }
 
 /**
@@ -203,6 +203,23 @@ log_score (gint sc)
   return pos;
 }
 
+/**
+ * kill_player
+ *
+ * Description:
+ * Ends the current game.
+ **/
+static void
+kill_player (void)
+{
+    game_state = STATE_DEAD;
+    play_sound (SOUND_DIE);
+    arena[player_xpos][player_ypos] = OBJECT_PLAYER;
+    endlev_counter = 0;
+    add_aieee_bubble (player_xpos, player_ypos);
+    player_animation_dead ();
+    set_move_menu_sensitivity (FALSE);
+}
 
 /**
  * add_kill
@@ -279,6 +296,28 @@ clear_arena (void)
   num_robots2 = 0;
 }
 
+/**
+ * load_temp_arena
+ *
+ * Description:
+ * Set up the temporary arena for processing speculative moves. 
+ *
+ **/
+static void
+load_temp_arena (void)
+{
+  gint i, j;
+
+  for (i = 0; i < GAME_WIDTH; ++i) {
+    for (j = 0; j < GAME_HEIGHT; ++j) {
+      if (arena[i][j] != OBJECT_PLAYER) {
+	temp_arena[i][j] = arena[i][j];
+      } else {
+	temp_arena[i][j] = OBJECT_NONE;
+      }
+    }
+  }
+}
 
 /**
  * check_location
@@ -459,15 +498,8 @@ update_arena (void)
     }
   }
 
-  if (arena[player_xpos][player_ypos] != OBJECT_PLAYER) {
-    game_state = STATE_DEAD;
-    play_sound (SOUND_DIE);
-    arena[player_xpos][player_ypos] = OBJECT_PLAYER;
-    endlev_counter = 0;
-    add_aieee_bubble (player_xpos, player_ypos);
-    player_animation_dead ();
-    set_move_menu_sensitivity (FALSE);
-  }
+  if (arena[player_xpos][player_ypos] != OBJECT_PLAYER)
+    kill_player ();
 
   if ((num_robots1 + num_robots2) <= 0) {
     game_state = STATE_COMPLETE;
@@ -920,7 +952,6 @@ push_heap (gint x, gint y, gint dx, gint dy)
 static gboolean
 try_player_move (gint dx, gint dy)
 {
-  gint i, j;
   gint nx, ny;
 
   nx = player_xpos + dx;
@@ -930,15 +961,7 @@ try_player_move (gint dx, gint dy)
     return FALSE;
   }
 
-  for (i = 0; i < GAME_WIDTH; ++i) {
-    for (j = 0; j < GAME_HEIGHT; ++j) {
-      if (arena[i][j] != OBJECT_PLAYER) {
-	temp_arena[i][j] = arena[i][j];
-      } else {
-	temp_arena[i][j] = OBJECT_NONE;
-      }
-    }
-  }
+  load_temp_arena ();
 
   if (temp_arena[nx][ny] == OBJECT_HEAP) {
     if (game_config ()->moveable_heaps) {
@@ -1032,6 +1055,8 @@ safe_teleport_available (void)
 {
   int x, y;
 
+  load_temp_arena ();
+
   for (y = 0; y < GAME_WIDTH; y++) {
     for (x = 0; x < GAME_HEIGHT; x++) {
       if (check_safe (x, y))
@@ -1179,18 +1204,13 @@ safe_teleport (void)
 {
   gint xp, yp, ixp, iyp;
   gint i, j;
-  GtkWidget *dialog;
 
   if (properties_super_safe_moves () && !safe_teleport_available ()) {
     /* FIXME: This code is untested - in normal play you have to get to 
      * about level 61. */
     if (!safe_move_available ()) {
-      dialog = gtk_message_dialog_new (GTK_WINDOW (app), GTK_DIALOG_MODAL, 
-				       GTK_MESSAGE_INFO, GTK_BUTTONS_OK, 
-				       _("You have run out of safe moves - the robots have won!"));
-      gtk_dialog_run (GTK_DIALOG (dialog));
-      /* This is a complete hack to force death. */
-      arena[player_xpos][player_ypos] = OBJECT_ROBOT1;
+      message_box (_("You have run out of safe moves - the robots have won!"));
+      kill_player ();
       return FALSE;
     } else if (safe_teleports > 0) {
       message_box (_("There are no safe locations to teleport to!!"));
@@ -1199,7 +1219,10 @@ safe_teleport (void)
       return random_teleport ();
     }
   }
-  
+
+  if (safe_teleports <= 0)
+    return random_teleport ();
+
   for (i = 0; i < GAME_WIDTH; ++i) {
     for (j = 0; j < GAME_HEIGHT; ++j) {
       if (arena[i][j] != OBJECT_PLAYER) {
