@@ -25,13 +25,14 @@
 #include <string.h>
 #include <gdk-pixbuf/gdk-pixbuf.h>
 
-#include "games-preimage.h"
-#include "games-gridframe.h"
+#include <games-preimage.h>
+#include <games-gridframe.h>
+#include <games-stock.h>
 
 #include "pieces.h"
 
 #define APPNAME "gnotski"
-#define APPNAME_LONG "GNOME Klotski"
+#define APPNAME_LONG "Klotski"
 
 #define MINWIDTH 250
 #define MINHEIGHT 250
@@ -45,11 +46,11 @@
 GConfClient *conf_client;
 
 GtkWidget *window;
-GtkWidget *menubar;
 GtkWidget *statusbar;
 GtkWidget *space;
-GtkWidget *move_value;
 GtkWidget *gameframe;
+GtkWidget *messagewidget;
+GtkWidget *moveswidget;
 
 GdkGC *space_gc = NULL;
 
@@ -66,6 +67,9 @@ gchar *map = NULL;
 gchar *tmpmap = NULL;
 gchar *move_map = NULL;
 gchar *orig_map = NULL;
+gchar *lastmove_map = NULL;
+gchar *undomove_map = NULL;
+
 gchar current_level_scorefile[4];
 
 gint space_width = 0;
@@ -83,31 +87,25 @@ guint redraw_all_idle_id = 0;
 guint configure_idle_id = 0;
 
 void create_space (void);
-void create_menubar (void);
+GtkWidget * create_menubar (void);
 void create_statusbar (void);
 
-gboolean redraw_all (void);
-void status_message (gchar *);
 void load_image (void);
 void gui_draw_space (void);
 void gui_draw_pixmap (char *, gint, gint);
 gint get_piece_nr (char *, gint, gint);
 gint get_piece_id (char *, gint, gint);
 void set_piece_id (char *, gint, gint, gint);
-gint check_valid_move (gint, gint, gint);
-gint do_move_piece (gint, gint, gint);
 gint move_piece (gint, gint, gint, gint, gint);
 void copymap (char *, char *);
 gint mapcmp (char *, char *);
-static gint save_state (GnomeClient *, gint, GnomeRestartStyle, gint,
+gint save_state (GnomeClient *, gint, GnomeRestartStyle, gint,
                         GnomeInteractStyle, gint fast, gpointer);
-void set_move (gint);
 void new_move (void);
 void game_score (void);
 gint game_over (void);
-void update_menu_state (void);
 
-static gboolean window_resize_cb (GtkWidget *, GdkEventConfigure *, gpointer);
+gboolean window_resize_cb (GtkWidget *, GdkEventConfigure *, gpointer);
 
 void new_game (gint requested_level);
 
@@ -117,7 +115,6 @@ void level_cb (GtkAction *, GtkRadioAction *);
 void quit_game_cb (GtkAction *);
 void restart_level_cb (GtkAction *);
 void help_cb (GtkAction *);
-void hint_cb (GtkAction *);
 void about_cb (GtkAction *);
 void score_cb (GtkAction *);
 
@@ -132,11 +129,11 @@ typedef struct _levelinfo {
   gchar *data;
 } levelinfo;
 
-/* The remarks below provide context for translation. */
+/* The "puzzle name" remarks provide context for translation. */
 const levelinfo level[]= {
      /* puzzle name */ 
   { N_("Only 18 Steps"), 0,
-    6, 9, 18, 
+    6, 9, 18,
     "######" \
     "#a**b#" \
     "#m**n#" \
@@ -212,7 +209,7 @@ const levelinfo level[]= {
     "    .." \
     "    .." },
 
-     /* puzzle name */
+     /* puzzle name - sometimes called "Le'Ane Rouge" */
   { N_("Red Donkey"), 0,
     6, 9, 81,
     "######" \
@@ -253,7 +250,7 @@ const levelinfo level[]= {
 
      /* puzzle name */
   { N_("Agatka"), 1,
-    7, 7, 0,
+    7, 7, 30,
     "..     " \
     ".      " \
     "#####--" \
@@ -264,7 +261,7 @@ const levelinfo level[]= {
 
      /* puzzle name */
   { N_("Success"), 1,
-    9, 6, 0,
+    9, 6, 25,
     "#######  " \
     "#**bbc#  " \
     "#defgh#  " \
@@ -274,7 +271,7 @@ const levelinfo level[]= {
 
      /* puzzle name */
   { N_("Bone"), 1,
-    6, 9, 0,
+    6, 9, 14,
     "######" \
     "#abc*#" \
     "# dd*#" \
@@ -287,7 +284,7 @@ const levelinfo level[]= {
 
      /* puzzle name */
   { N_("Fortune"), 1,
-    7, 10, 0,
+    7, 10, 25,
     "     .." \
     "     . " \
     "####-. " \
@@ -301,7 +298,7 @@ const levelinfo level[]= {
 
      /* puzzle name */
   { N_("Fool"), 1,
-    10, 6, 0,
+    10, 6, 29,
     "  ########" \
     "  -aabc  #" \
     "  #aabdef#" \
@@ -311,20 +308,20 @@ const levelinfo level[]= {
 
      /* puzzle name */
   { N_("Solomon"), 1,
-    7, 9, 0,
+    7, 9, 29, 
     " .     " \
     "..     " \
     "#--####" \
     "#  aab#" \
     "# cdfb#" \
-    "-hcefg#" \
+    "#hcefg#" \
     "#hijk*#" \
     "#hll**#" \
     "#######" },
 
      /* puzzle name */
   { N_("Cleopatra"), 1,
-    6, 8, 0,
+    6, 8, 32, 
     "######" \
     "#abcd#" \
     "#**ee#" \
@@ -336,7 +333,7 @@ const levelinfo level[]= {
 
      /* puzzle name */
   { N_("Shark"), 1,
-    11, 8, 0,
+    11, 8, 0, /* SOLVEME */
     "########   " \
     "#nrr s #   " \
     "#n*op q#   " \
@@ -348,15 +345,15 @@ const levelinfo level[]= {
 
      /* puzzle name */
   { N_("Rome"), 1,
-    8, 8, 0,
+    8, 8, 38, 
     "########" \
     "#abcc**#" \
     "#ddeef*#" \
     "#ddghfi#" \
-    "-   jki#" \
+    "#   jki#" \
     "#--#####" \
-    "      .." \
-    "       ." },
+    " ..     " \
+    "  .     " },
 
      /* puzzle name */
   { N_("Pennant Puzzle"), 1,
@@ -373,7 +370,7 @@ const levelinfo level[]= {
 
      /* puzzle name */
   { N_("Ithaca"), 2,
-    19, 19, 0,
+    19, 19, 0, /* SOLVEME */
     ".aaaaaaaaaaaaaaaaab" \
     "..  cddeffffffffffb" \
     " .. cddeffffffffffb" \
@@ -396,22 +393,20 @@ const levelinfo level[]= {
 
      /* puzzle name */
   { N_("Pelopones"), 2,
-    9, 10, 0,
+    9, 8, 0, /* SOLVEME */
     "#########" \
     "#abbb***#" \
     "#abbb*c*#" \
     "#adeefgg#" \
     "#  eefhh#" \
-    "#    ihh#" \
-    "#    ihh#" \
-    "#---#####" \
-    "      ..." \
-    "      . ." },
+    "#... ihh#" \
+    "#. . ihh#" \
+    "#########" },
 
      /* puzzle name */
   { N_("Transeuropa"), 2,
-    15, 8, 0,
-    "    -##-#######" \
+    15, 8, 0, /* SOLVEME */
+    "    ###########" \
     "    -AAAAABBCC#" \
     "    -   DEFGHI#" \
     "    #   DEFGJI#" \
@@ -422,36 +417,32 @@ const levelinfo level[]= {
 
      /* puzzle name */
   { N_("Lodzianka"), 2,
-    11, 7, 0,
-    "#########  " \
-    "#**abbcc#  " \
-    "#**abbdd#  " \
-    "#eefgh  #  " \
-    "#iiijk  -  " \
-    "#iiijk  -.." \
-    "#########.." },
+    9, 7, 0, /* SOLVEME */
+    "#########" \
+    "#**abbcc#" \
+    "#**abbdd#" \
+    "#eefgh  #" \
+    "#iiijk..#" \
+    "#iiijk..#" \
+    "#########" },
 
      /* puzzle name */
   { N_("Polonaise"), 2,
-    7, 9, 0,
+    7, 7, 0, /* SOLVEME */
     "#######" \
     "#aab**#" \
     "#aabc*#" \
-    "-defgg#" \
-    "#  fhh#" \
-    "#  ihh#" \
-    "#--####" \
-    "     .." \
-    "      ." },
+    "#defgg#" \
+    "#..fhh#" \
+    "# .ihh#" \
+    "#######" },
 
      /* puzzle name */
   { N_("Baltic Sea"), 2,
-    6, 10, 0,
-    ".     " \
-    ".     " \
-    "#-####" \
-    "# abc#" \
-    "# dec#" \
+    6, 8, 42,
+    "######" \
+    "#.abc#" \
+    "#.dec#" \
     "#fggc#" \
     "#fhhi#" \
     "#fjk*#" \
@@ -460,7 +451,7 @@ const levelinfo level[]= {
 
      /* puzzle name */
   { N_("American Pie"), 2,
-    10, 12, 0,
+    10, 12, 0, /* SOLVEME */
     "##########" \
     "#a*bcdefg#" \
     "#**bhhhhg#" \
@@ -487,42 +478,29 @@ const levelinfo level[]= {
 
      /* puzzle name */
   { N_("Sunshine"), 2,
-    29, 35, 0,
-    "           #######           " \
-    "           # ... #           " \
-    "           #.. ..#           " \
-    "           #.   .#           " \
-    "           #.. ..#           " \
-    "           # ... #           " \
-    "############-----############" \
-    "#\x90\x90\x91\x92\x92\x80\x80\x80\x80\x80 \xb0\xb0\xb1\xb2\xb2 \x81\x81\x81\x81\x81\x98\x98\x99\x9a\x9a#" \
-    "#\x90\x90\x91\x92\x92\x80\x80\x80\x80\x80 \xb0\xb0\xb1\xb2\xb2 \x81\x81\x81\x81\x81\x98\x98\x99\x9a\x9a#" \
-    "#\x93\x93 \x94\x94\x80\x80\x80\x80\x80 \xb3\xb3 \xb4\xb4 \x81\x81\x81\x81\x81\x9b\x9b \x9c\x9c#" \
-    "#\x95\x95\x96\x97\x97\x80\x80\x80\x80\x80 \xb5\xb5\xb6\xb7\xb7 \x81\x81\x81\x81\x81\x9d\x9d\x9e\x9f\x9f#" \
-    "#\x95\x95\x96\x97\x97\x80\x80\x80\x80\x80 \xb5\xb5\xb6\xb7\xb7 \x81\x81\x81\x81\x81\x9d\x9d\x9e\x9f\x9f#" \
-    "#\x84\x84\x84\x84\x84#################\x83\x83\x83\x83\x83#" \
-    "#\x84\x84\x84\x84\x84#ddFeeA***BffOZZ#\x83\x83\x83\x83\x83#" \
-    "#\x84\x84\x84\x84\x84#ddFee** **ffOZZ#\x83\x83\x83\x83\x83#" \
-    "#\x84\x84\x84\x84\x84#MMKQQ*   *PPS\xde\xde#\x83\x83\x83\x83\x83#" \
-    "#\x84\x84\x84\x84\x84#VVLXX** **bbRcc#\x83\x83\x83\x83\x83#" \
-    "#     #VVLXXD***EbbRcc#     #" \
-    "#\xc0\xc0\xc1\xc2\xc2#\x89\x89\x89\x89\x89TTJWW\x8a\x8a\x8a\x8a\x8a#\xc8\xc8\xc9\xca\xca#" \
-    "#\xc0\xc0\xc1\xc2\xc2#\x89\x89\x89\x89\x89TTJWW\x8a\x8a\x8a\x8a\x8a#\xc8\xc8\xc9\xca\xca#" \
-    "#\xc3\xc3 \xc4\xc4#\x89\x89\x89\x89\x89GG HH\x8a\x8a\x8a\x8a\x8a#\xcb\xcb \xcc\xcc#" \
-    "#\xc5\xc5\xc6\xc7\xc7#\x89\x89\x89\x89\x89YYIgg\x8a\x8a\x8a\x8a\x8a#\xcd\xcd\xce\xcf\xcf#" \
-    "#\xc5\xc5\xc6\xc7\xc7#\x89\x89\x89\x89\x89YYIgg\x8a\x8a\x8a\x8a\x8a#\xcd\xcd\xce\xcf\xcf#" \
-    "#     #hh\xd0iilltmmpp\xd1qq#     #" \
-    "#\x85\x85\x85\x85\x85#hh\xd2iill mmpp\xddqq#\x86\x86\x86\x86\x86#" \
-    "#\x85\x85\x85\x85\x85#2y\xd3\xd4\xd5v s w\xd8\xd9x\xdaz#\x86\x86\x86\x86\x86#" \
-    "#\x85\x85\x85\x85\x85#jj\xd6kkaa nnoo\xdbrr#\x86\x86\x86\x86\x86#" \
-    "#\x85\x85\x85\x85\x85#jj\xd7kkaaunnoo\xdcrr#\x86\x86\x86\x86\x86#" \
-    "#\x85\x85\x85\x85\x85######-----######\x86\x86\x86\x86\x86#" \
-    "#\xa0\xa0\xa1\xa2\xa2\x88\x88\x88\x88\x88 \xb8\xb8\xb9\xba\xba \x87\x87\x87\x87\x87\xa8\xa8\xa9\xaa\xaa#" \
-    "#\xa0\xa0\xa1\xa2\xa2\x88\x88\x88\x88\x88 \xb8\xb8\xb9\xba\xba \x87\x87\x87\x87\x87\xa8\xa8\xa9\xaa\xaa#" \
-    "#\xa3\xa3 \xa4\xa4\x88\x88\x88\x88\x88 \xbb\xbb \xbc\xbc \x87\x87\x87\x87\x87\xab\xab \xac\xac#" \
-    "#\xa5\xa5\xa6\xa7\xa7\x88\x88\x88\x88\x88 \xbd\xbd\xbe\xbf\xbf \x87\x87\x87\x87\x87\xad\xad\xae\xaf\xaf#" \
-    "#\xa5\xa5\xa6\xa7\xa7\x88\x88\x88\x88\x88 \xbd\xbd\xbe\xbf\xbf \x87\x87\x87\x87\x87\xad\xad\xae\xaf\xaf#" \
-    "#############################" },
+    17, 22, 345,
+    "       ...       " \
+    "      .. ..      " \
+    "      .   .      " \
+    "      .. ..      " \
+    "       ...       " \
+    "######-----######" \
+    "#hh0iilltmmpp;qq#" \
+    "#hh,iill mmpp:qq#" \
+    "#2y{45v s w89x/z#" \
+    "#jj6kkaa nnoo<rr#" \
+    "#jj7kkaaunnoo>rr#" \
+    "#33333TTJWW11111#" \
+    "#33333TTJWW11111#" \
+    "#33333GG HH11111#" \
+    "#33333YYIgg11111#" \
+    "#33333YYIgg11111#" \
+    "#ddFeeA***BffOZZ#" \
+    "#ddFee** **ffOZZ#" \
+    "#MMKQQ*   *PPS^^#" \
+    "#VVLXX** **bbRcc#" \
+    "#VVLXXD***EbbRcc#" \
+    "#################" },
 
      /* puzzle name */
   { N_("Block 10"), 3,
@@ -659,24 +637,22 @@ const GtkActionEntry entries[] = {
   { "SkillPack", NULL, N_("Skill Pack") },
                           /* set of puzzles */
   { "MinoruClimb", NULL, N_("Minoru Climb") },
-  { "RestartPuzzle", NULL, N_("_Restart Puzzle"), "<control>R", NULL, G_CALLBACK (restart_level_cb) },
-  { "NextPuzzle", NULL, N_("Next Puzzle"), NULL, NULL, G_CALLBACK (next_level_cb) },
-  { "PrevPuzzle", NULL, N_("Previous Puzzle"), NULL, NULL, G_CALLBACK (prev_level_cb) },
-  { "Hint", NULL, N_("_Hint"), NULL, NULL, G_CALLBACK (hint_cb) },
+  { "RestartPuzzle", GTK_STOCK_REFRESH, N_("_Restart Puzzle"), "<control>R", NULL, G_CALLBACK (restart_level_cb) },
+  { "NextPuzzle", GTK_STOCK_GO_FORWARD, N_("Next Puzzle"), "Page_Down", NULL, G_CALLBACK (next_level_cb) },
+  { "PrevPuzzle", GTK_STOCK_GO_BACK, N_("Previous Puzzle"), "Page_Up", NULL, G_CALLBACK (prev_level_cb) },
   { "Quit", GTK_STOCK_QUIT,  NULL, NULL, NULL, G_CALLBACK (quit_game_cb) },
-  { "Contents", GTK_STOCK_HELP, "_Contents", "F1", NULL, G_CALLBACK (help_cb) },
+  { "Contents", GAMES_STOCK_CONTENTS, NULL, NULL, NULL, G_CALLBACK (help_cb) },
   { "About", GTK_STOCK_ABOUT, NULL, NULL, NULL, G_CALLBACK (about_cb) },
-  { "Scores", GNOME_STOCK_SCORES, N_("_Scores"), NULL, NULL, G_CALLBACK (score_cb) }
+  { "Scores", GAMES_STOCK_SCORES, NULL, NULL, NULL, G_CALLBACK (score_cb) }
 };
 
-const char *ui_description =
+const char ui_description[] =
 "<ui>"
 "  <menubar name='MainMenu'>"
 "    <menu action='GameMenu'>"
 "      <menuitem action='RestartPuzzle'/>"
 "      <menuitem action='NextPuzzle'/>"
 "      <menuitem action='PrevPuzzle'/>"
-"      <menuitem action='Hint'/>"
 "      <separator/>"
 "      <menu action='HuaRongTrail'/>"
 "      <menu action='ChallengePack'/>"
@@ -704,23 +680,25 @@ static const struct poptOption options[] = {
 
 /* ------------------------------------------------------- */
 
-int 
+int
 main (int argc, char **argv)
 {
   GnomeClient *client;
   GtkWidget *vbox;
+  GtkWidget *menubar;
   gint win_width, win_height, startup_level;
 
   gnome_score_init (APPNAME);
   bindtextdomain (GETTEXT_PACKAGE, GNOMELOCALEDIR);
   bind_textdomain_codeset (GETTEXT_PACKAGE, "UTF-8");
   textdomain (GETTEXT_PACKAGE);
-   
+  
   gnome_program_init (APPNAME, VERSION,
                        LIBGNOMEUI_MODULE,
                        argc, argv,
                        GNOME_PARAM_POPT_TABLE, options,
                        GNOME_PARAM_APP_DATADIR, DATADIR, NULL);
+  games_stock_init ();
   gnome_window_icon_set_default_from_file (GNOME_ICONDIR"/gnotski-icon.png");
   client = gnome_master_client ();
   g_object_ref (G_OBJECT (client));
@@ -749,13 +727,14 @@ main (int argc, char **argv)
   gnome_app_set_contents (GNOME_APP (window), vbox);
 
   load_image ();
-  create_space (); 
-  create_menubar ();
+  create_space ();
+  menubar = create_menubar ();
   create_statusbar ();
 
   gtk_box_pack_start (GTK_BOX (vbox), menubar, FALSE, FALSE, 0);
   gtk_box_pack_start (GTK_BOX (vbox), gameframe, TRUE, TRUE, 0);
-  gtk_box_pack_end (GTK_BOX (vbox), statusbar, FALSE, FALSE, 0);
+  gtk_box_pack_start (GTK_BOX (vbox), gtk_hseparator_new (), FALSE, FALSE, 0);
+  gtk_box_pack_end (GTK_BOX (vbox), statusbar, FALSE, FALSE, GNOME_PAD);
  
   if (session_xpos >= 0 && session_ypos >= 0)
     gtk_window_move (GTK_WINDOW (window), session_xpos, session_ypos);
@@ -765,6 +744,8 @@ main (int argc, char **argv)
   new_game (startup_level);
   
   gtk_main ();
+
+  gnome_accelerators_sync();
   
   return 0;
 }
@@ -783,7 +764,7 @@ expose_space (GtkWidget *widget, GdkEventExpose *event)
   return FALSE; 
 }
 
-gboolean
+static gboolean
 redraw_all (void)
 {
   gint x, y;
@@ -835,7 +816,7 @@ button_release_space (GtkWidget *widget, GdkEventButton *event)
 	if (mapcmp (move_map, map)) {
 	  new_move ();
 	  if (game_over ()) {
-	    status_message (_("Level completed. Well done."));
+	    gtk_label_set_text (GTK_LABEL (messagewidget), _("Level completed."));
 	    game_score ();
 	  }
 	}
@@ -988,12 +969,12 @@ update_score_state (void)
   top = gnome_score_get_notable (APPNAME, current_level_scorefile,
                                  &names, &scores, &scoretimes);
   if (top > 0) {
-    g_object_set (score_action, "sensitive", TRUE, NULL);
+    gtk_action_set_sensitive (score_action, TRUE);
     g_strfreev (names);
     g_free (scores);
     g_free (scoretimes);
   } else {
-    g_object_set (score_action, "sensitive", FALSE, NULL);
+    gtk_action_set_sensitive (score_action, FALSE);
   }
 }
 
@@ -1022,11 +1003,14 @@ configure_pixmaps_idle (void)
     }
 
     if (tiles_pixbuf == NULL) {
-      /* FIXME: Add an error dialog for corrupt images or bad loader. */
-      tiles_pixbuf = gdk_pixbuf_new (GDK_COLORSPACE_RGB, TRUE, 8,
-                                     tile_size * THEME_TILE_SEGMENTS, 
-                                     tile_size * 2);
-      gdk_pixbuf_fill (tiles_pixbuf, 0x00000000);
+      GtkWidget *dialog;
+      dialog = gtk_message_dialog_new (GTK_WINDOW (window),
+                                       GTK_DIALOG_MODAL,
+                                       GTK_MESSAGE_ERROR,
+                                       GTK_BUTTONS_OK,
+                                       _("The theme for this game failed to render.\n\nPlease check that Klotski is installed correctly."));
+      gtk_dialog_run (GTK_DIALOG (dialog));
+      exit (1);
     }
 
     prior_tile_size = tile_size;
@@ -1141,7 +1125,7 @@ void add_puzzle_menu (GtkUIManager *ui_manager)
 			 NULL, 0);
 }
 
-void
+GtkWidget *
 create_menubar (void)
 {
   GtkUIManager *ui_manager;
@@ -1160,34 +1144,20 @@ create_menubar (void)
   accel_group = gtk_ui_manager_get_accel_group (ui_manager);
   gtk_window_add_accel_group (GTK_WINDOW (window), accel_group);
   
-  menubar = gtk_ui_manager_get_widget (ui_manager, "/MainMenu");
+  return gtk_ui_manager_get_widget (ui_manager, "/MainMenu");
 }
 
 void
 create_statusbar (void)
 {
-  GtkWidget *frame, *move_box, *move_label;
+  statusbar = gtk_hbox_new (TRUE, 0);
 
-  statusbar = gtk_statusbar_new ();
-  gtk_statusbar_set_has_resize_grip (GTK_STATUSBAR (statusbar), FALSE);
+  messagewidget = gtk_label_new ("");
+  gtk_box_pack_start (GTK_BOX (statusbar), messagewidget, FALSE, FALSE, 0);
 
-  move_box = gtk_hbox_new (FALSE, 0);
-  move_label = gtk_label_new (_("Moves:"));
-  gtk_box_pack_start (GTK_BOX (move_box), move_label, FALSE, FALSE, 6);
-  move_value = gtk_label_new ("000");
-  gtk_box_pack_start (GTK_BOX (move_box), move_value, FALSE, FALSE, 6);
 
-  frame = gtk_frame_new (NULL);
-  gtk_frame_set_shadow_type (GTK_FRAME (frame), GTK_SHADOW_IN);
-  gtk_container_add (GTK_CONTAINER (frame), move_box);
-  gtk_box_pack_end (GTK_BOX (statusbar),frame, FALSE, FALSE, 0);
-}
-
-void
-status_message (gchar *message)
-{
-  gtk_statusbar_pop (GTK_STATUSBAR (statusbar), 0);
-  gtk_statusbar_push (GTK_STATUSBAR (statusbar), 0, message);
+  moveswidget = gtk_label_new ("");
+  gtk_box_pack_end (GTK_BOX (statusbar), moveswidget, FALSE, FALSE, 0);
 }
 
 void
@@ -1206,7 +1176,7 @@ load_image (void)
                                      GTK_DIALOG_MODAL,
                                      GTK_MESSAGE_ERROR,
                                      GTK_BUTTONS_OK,
-                                     _("Could not find the image: \n%s\n\nPlease check your gnome-games installation."),
+                                     _("Could not find the image:\n%s\n\nPlease check that Klotski is installed correctly."),
                                      fname);
     gtk_dialog_run (GTK_DIALOG (dialog));
     exit (1);
@@ -1214,7 +1184,7 @@ load_image (void)
   g_free (fname);
 }
 
-void
+static void
 set_move (gint x)
 {
   moves = x - 1;
@@ -1224,11 +1194,29 @@ set_move (gint x)
 void
 new_move (void)
 {
+  static gint last_piece_id = -2;
   gchar *str = NULL;
-  if (moves < 999)
-    moves++;
-  str = g_strdup_printf ("%03d", moves);
-  gtk_label_set_text (GTK_LABEL (move_value), str);
+
+  if (moves < 1)
+	last_piece_id = -2;
+
+  if (last_piece_id != piece_id) {
+    copymap (undomove_map, lastmove_map);
+    if (moves < 999) 
+      moves++;
+  }
+
+  if ((moves > 0) && !mapcmp(undomove_map,map)) {    
+    moves--;
+    last_piece_id = -2;
+  } else {
+    last_piece_id = piece_id;
+  }
+
+  copymap (lastmove_map, map);
+
+  str = g_strdup_printf (_("Moves: %d"), moves);
+  gtk_label_set_text (GTK_LABEL (moveswidget), str);
   g_free (str);
 }
 
@@ -1244,7 +1232,7 @@ game_over (void)
   return over;
 }
 
-gint
+static gint
 do_move_piece (gint id, gint dx, gint dy)
 {
   gint x, y;
@@ -1282,7 +1270,7 @@ do_move_piece (gint id, gint dx, gint dy)
   return 1;
 }
 
-gint
+static gint
 check_valid_move (gint id, gint dx, gint dy)
 {
   gint x, y, valid = 1;
@@ -1393,24 +1381,20 @@ static void
 prepare_map (current_level)
 {
   gint x, y = 0;
-  gchar *tmp; 
   gchar *leveldata;
 
   leveldata = level[current_level].data; 
   width = level[current_level].width;
   height = level[current_level].height;
-  tmp = g_strdup_printf (_("Level %d : %s"), 
-                         current_level + 1, 
-                         level[current_level].name);
-  
-  status_message (tmp);
-  g_free (tmp);
+  gtk_label_set_text (GTK_LABEL (messagewidget), _(level[current_level].name));
 
   if (map) {
     free (map);
     free (tmpmap);
     free (move_map);
     free (orig_map);
+    free (lastmove_map);
+    free (undomove_map);
   }
 
   piece_id = -1;
@@ -1420,13 +1404,35 @@ prepare_map (current_level)
   tmpmap = calloc (1, (width + 2) * (height + 2));
   orig_map = calloc (1, (width + 2) * (height + 2));
   move_map = calloc (1, (width + 2) * (height + 2));
+  lastmove_map = calloc (1, (width + 2) * (height + 2));
+  undomove_map = calloc (1, (width + 2) * (height + 2));
   if (leveldata)
     for (y = 0; y < height; y++)
       for (x = 0; x < width; x++)
         set_piece_id (map, x, y, *leveldata++);
   copymap (orig_map, map);
+  copymap (lastmove_map, map);
 }
 
+static void
+update_menu_state (void)
+{
+  GtkAction *action;
+  gboolean action_is_sensitive;
+
+  /* Puzzle Radio Action */
+  gtk_toggle_action_set_active (level_action[current_level], TRUE);
+
+  /* Next Puzzle Sensitivity */
+  action_is_sensitive = current_level < max_level;
+  action = gtk_action_group_get_action (action_group, "NextPuzzle");
+  gtk_action_set_sensitive (action, action_is_sensitive);
+
+  /* Previous Puzzle Sensitivity */
+  action_is_sensitive = current_level > 0;
+  action = gtk_action_group_get_action (action_group, "PrevPuzzle");
+  gtk_action_set_sensitive (action, action_is_sensitive);
+}
 
 void
 new_game (gint requested_level)
@@ -1454,7 +1460,7 @@ quit_game_cb (GtkAction *action)
   gtk_main_quit ();
 }
 
-static gboolean
+gboolean
 window_resize_cb (GtkWidget *w, GdkEventConfigure *e, gpointer data)
 {
   gconf_client_set_int (conf_client, "/apps/gnotski/width",
@@ -1464,7 +1470,7 @@ window_resize_cb (GtkWidget *w, GdkEventConfigure *e, gpointer data)
   return FALSE;
 }
 
-static gint
+gint
 save_state (GnomeClient *client, gint phase, 
             GnomeRestartStyle save_style, gint shutdown,
             GnomeInteractStyle interact_style, gint fast,
@@ -1523,53 +1529,6 @@ help_cb (GtkAction *action)
   gnome_help_display ("gnotski.xml", NULL, NULL);
 }
 
-void 
-update_menu_state (void)
-{
-  GtkAction *action;
-  gboolean action_is_sensitive; 
-
-  /* Puzzle Radio Action */
-  gtk_toggle_action_set_active (level_action[current_level], TRUE);
-
-  /* Hint Sensitivity */
-  action_is_sensitive = level[current_level].minimum_moves > 0;
-  action = gtk_action_group_get_action (action_group, "Hint");
-  g_object_set (action, "sensitive", action_is_sensitive, NULL); 
-
-  /* Next Puzzle Sensitivity */
-  action_is_sensitive = current_level < max_level;
-  action = gtk_action_group_get_action (action_group, "NextPuzzle");
-  g_object_set (action, "sensitive", action_is_sensitive, NULL);
-
-  /* Previous Puzzle Sensitivity */
-  action_is_sensitive = current_level > 0;
-  action = gtk_action_group_get_action (action_group, "PrevPuzzle");
-  g_object_set (action, "sensitive", action_is_sensitive, NULL);
-}
-
-void
-hint_cb (GtkAction *action)
-{
-  GtkWidget *hint_dlg;
-  char *message;
-
-  message = g_strdup_printf (ngettext ("This puzzle is solvable in %d move.",
-                                       "This puzzle is solvable in %d moves.",
-                                       level[current_level].minimum_moves),
-                                       level[current_level].minimum_moves);
-  
-  hint_dlg = gtk_message_dialog_new (GTK_WINDOW (window),
-                                     GTK_DIALOG_DESTROY_WITH_PARENT,
-                                     GTK_MESSAGE_INFO,
-                                     GTK_BUTTONS_OK,
-                                     message);
-
-  gtk_dialog_run (GTK_DIALOG (hint_dlg));
-  gtk_widget_destroy (hint_dlg);
-  g_free (message);
-}
-
 void
 about_cb (GtkAction *action)
 {
